@@ -5,6 +5,7 @@
 @author: felix
 """
 from functools import partial
+
 import strongtyping
 
 
@@ -19,16 +20,26 @@ def action(f, frefs):
     else:
         action.f.update({}.fromkeys(action.f, None))  # reset all values to None
         action.qualname = f.__qualname__
-    action.f.update({}.fromkeys(frefs.split('_'), f))  # set all frefs values to f
+    action.f.update({}.fromkeys(frefs.split("_"), f))  # set all frefs values to f
 
     # this line was added by myself
-    action.f['setter'] = strongtyping.strong_typing.match_typing(action.f['setter']) if action.f['setter'] is not None else None
+    action.f["setter"] = (
+        strongtyping.strong_typing.match_typing(action.f["setter"])
+        if action.f["setter"] is not None
+        else None
+    )
 
-    return property(*(action.f[ref] if (ref != "documenter" or action.f[ref] is None)
-                      else action.f[ref](0) for ref in action.f))
+    return property(
+        *(
+            action.f[ref] if (ref != "documenter" or action.f[ref] is None) else action.f[ref](0)
+            for ref in action.f
+        )
+    )
 
 
 action.qualname = None
 action.f = dict.fromkeys(["getter", "setter", "deleter", "documenter"], None)
 
-globals().update({fref: partial(action, frefs=fref) for fref in {**action.f, 'getter_setter': None}})
+globals().update(
+    {fref: partial(action, frefs=fref) for fref in {**action.f, "getter_setter": None}}
+)
