@@ -6,8 +6,9 @@
 """
 import inspect
 import os
-import sys
 import typing
+from collections import deque
+from collections.abc import Callable, Iterable
 from functools import lru_cache, partial
 from queue import Queue
 from typing import Any, TypeVar, _GenericAlias, _SpecialForm, _type_repr  # type: ignore
@@ -26,6 +27,7 @@ except ImportError as e:
 else:
     extension_module = bool(int(os.environ["ST_MODULES_INSTALLED"]))
 
+
 empty = object()
 default_return_queue = Queue()
 
@@ -42,11 +44,7 @@ class ValidationError(Exception):
         print(message)
 
 
-py_version = sys.version_info.minor
-if hasattr(typing, "_GenericAlias"):
-    typing_base_class = typing._GenericAlias  # type: ignore
-else:
-    typing_base_class = typing.GenericMeta  # type: ignore
+typing_base_class = typing._GenericAlias  # type: ignore
 
 
 @lru_cache(maxsize=1024)
@@ -76,7 +74,7 @@ def get_possible_types(typ_to_check, origin_name: str = "") -> typing.Union[tupl
 
 
 def get_origins(typ_to_check: Any) -> tuple:
-    from strongtyping.strong_typing import match_class_typing
+    from strongtyping.strong_typing import MatchTypedDict
 
     """
     :param typ_to_check: typ_to_check: some typing like List[str], Dict[str, int], Tuple[Union[str, int], List[int]]
@@ -86,21 +84,25 @@ def get_origins(typ_to_check: Any) -> tuple:
         - Tuple[Union[str, int], List[int]] = (tuple, 'Tuple)
         - FunctionType = (None, 'None')
     """
-    origin = None
+    origin, origin_name = None, ""
+
+    if isinstance(typ_to_check, MatchTypedDict):
+        return typ_to_check, typ_to_check.__class__.__name__
 
     if hasattr(typ_to_check, "__annotations__") and hasattr(typ_to_check, "__orig_bases__"):
-        return typ_to_check, typ_to_check.__orig_bases__[0].__name__
+        if typ_to_check.__orig_bases__:
+            orig_base = typ_to_check.__orig_bases__[0].__name__
+        else:
+            orig_base = typ_to_check.cls.__class__.__name__
+        return typ_to_check, orig_base
 
-    if hasattr(typ_to_check, "__annotations__") and hasattr(typ_to_check, "__total__"):
+    if typing.is_typeddict(typ_to_check):
         return typ_to_check, typ_to_check.__class__.__name__
 
     if hasattr(typ_to_check, "__origin__") or hasattr(typ_to_check, "__orig_bases__"):
-        if (
-            py_version >= 9
-            and hasattr(typ_to_check, "__origin__")
-            and hasattr(typ_to_check.__origin__, "__name__")
-        ):
-            origin = typ_to_check.__origin__.__name__
+        if hasattr(typ_to_check, "__origin__") and hasattr(typ_to_check.__origin__, "__name__"):
+            origin = typ_to_check.__origin__
+            origin_name = origin.__name__
         elif hasattr(typ_to_check, "__origin__") or hasattr(typ_to_check, "__orig_bases__"):
             if hasattr(typ_to_check, "__origin__") and typ_to_check.__origin__ is not None:
                 origin = typ_to_check.__origin__
@@ -111,9 +113,7 @@ def get_origins(typ_to_check: Any) -> tuple:
         origin_name = origin._name  # type: ignore
     else:
         if hasattr(typ_to_check, "_name"):
-            origin_name = typ_to_check._name
-        else:
-            origin_name = str(origin).replace("typing.", "")
+            origin_name = typ_to_check._name if typ_to_check._name is not None else origin_name
     return origin, origin_name
 
 
@@ -164,8 +164,6 @@ def checking_typing_type(arg: Any, possible_types: tuple, *args, **kwargs):
 def checking_typing_union(arg: Any, possible_types: tuple, mro, **kwargs):
     if mro:
         return any(pssble_type in arg for pssble_type in possible_types)
-    if isinstance(possible_types, typing_base_class):
-        return any(check_type(arg, typ, **kwargs) for typ in get_possible_types(possible_types))
     try:
         is_instance = isinstance(arg, possible_types)
     except TypeError:
@@ -177,29 +175,19 @@ def checking_typing_union(arg: Any, possible_types: tuple, mro, **kwargs):
             return validate_object(arg, kwargs.get("validation_with"))
 
 
+def checking_typing_optional(arg: Any, possible_types: tuple, mro, **kwargs):
+    return arg is None or check_type(arg, possible_types[0])
+
+
 def checking_typing_iterator(arg: Any, *args, **kwargs):
     return hasattr(arg, "__iter__") and hasattr(arg, "__next__")
 
 
 def checking_typing_callable(arg: Any, possible_types: tuple, *args, **kwargs):
-    def callable_check(parameter_type: object, required_parameter_type: object) -> bool:
-        if required_parameter_type == Ellipsis:
-            return True
-        else:
-            return parameter_type == required_parameter_type
-
     insp = inspect.signature(arg)
-    *required_params, return_val = possible_types
-    _, return_name = get_origins(return_val)
-    if return_name.lower() == "any":
-        correct_return_val = True
-    else:
-        correct_return_val = insp.return_annotation == return_val
+    return_val = insp.return_annotation == possible_types[-1]
     params = insp.parameters
-    return correct_return_val and all(
-        callable_check(param.annotation, required_param)
-        for param, required_param in zip(params.values(), required_params)
-    )
+    return return_val and all(p.annotation == pt for p, pt in zip(params.values(), possible_types))
 
 
 def checking_typing_tuple(arg: Any, possible_types: tuple, *args, **kwargs):
@@ -246,13 +234,6 @@ def checking_typing_literal(arg, possible_types, *args, **kwargs):
     return arg in possible_types
 
 
-def checking_typing__validator(arg, possible_types, *args, **kwargs):
-    """
-    required to support python versions 3.7, 3.8
-    """
-    return checking_typing_validator(arg, possible_types, *args, **kwargs)
-
-
 def checking_typing_validator(arg, possible_types, *args, **kwargs):
     if len(possible_types) == 2:
         default_return = empty
@@ -279,13 +260,6 @@ def checking_typing_validator(arg, possible_types, *args, **kwargs):
         return check_type(arg, required_type, **kwargs)
 
 
-def checking_typing__itervalidator(arg, possible_types, *args, **kwargs):
-    """
-    required to support python versions 3.7, 3.8
-    """
-    return checking_typing_itervalidator(arg, possible_types, *args, **kwargs)
-
-
 def checking_typing_itervalidator(arg, possible_types, *args, **kwargs):
     required_type, validation = possible_types
     return check_type(arg, required_type, validation_with=validation, **kwargs)
@@ -300,7 +274,7 @@ def checking_typing_iterable(arg: Any, possible_types: tuple, *args, **kwargs):
 
 def checking_typing_typedict_values(args: dict, required_types: dict, total: bool):
     if total:
-        return all(check_type(args[key], val) for key, val in required_types.items())
+        return all(check_type(args.get(key), val) for key, val in required_types.items())
     fields_to_check = {key: val for key, val in required_types.items() if key in args}
     return all(check_type(args[key], val) for key, val in fields_to_check.items())
 
@@ -318,6 +292,16 @@ def checking_typing_typeddict(arg: Any, possible_types: Any, *args, **kwargs):
     return checking_typing_typedict_values(arg, required_fields, total)
 
 
+def checking_typing_typeddict_required(arg: Any, possible_types: Any, *args, **kwargs):
+    return check_type(arg, possible_types[0])
+
+
+def checking_typing_typeddict_notrequired(arg: Any, possible_types: Any, *args, **kwargs):
+    if arg is None:
+        return True
+    return check_type(arg, possible_types[0])
+
+
 def module_checking_typing_list(arg: Any, possible_types: Any):
     if (
         not hasattr(possible_types, "__args__")
@@ -399,14 +383,11 @@ def check_duck_typing(arg, possible_types, *args, **kwargs):
 
 
 supported_typings = vars()
-if extension_module:
-    m = [f"module_checking_typing_{t}" for t in ("list", "dict", "set", "tuple", "validator")]
-    supported_modules = {k: v for k, v in vars().items() if k in m}
-else:
-    supported_modules = {}
 
 
 def check_type(argument, type_of, mro=False, **kwargs):
+    from strongtyping.types import IterValidator, Validator
+
     # if int(py_version) >= 10 and isinstance(type_of, (str, bytes)):
     #     type_of = eval(type_of, locals(), globals())
     if checking_typing_generator(argument, type_of):
@@ -418,16 +399,6 @@ def check_type(argument, type_of, mro=False, **kwargs):
         origin, origin_name = get_origins(type_of)
         origin_name = origin_name.lower()
 
-        if "any" in origin_name:
-            return validate_object(argument, kwargs.get("validation_with"))
-        if "json" in origin_name or "json" in str(type_of):
-            return supported_typings["checking_typing_json"](argument, type_of, mro)
-
-        try:
-            return supported_modules[f"module_checking_typing_{origin_name}"](argument, type_of)
-        except KeyError:
-            pass
-
         if "new_type" in origin_name:
             type_of = type_of.__supertype__
             origin, origin_name = get_origins(type_of)
@@ -436,17 +407,85 @@ def check_type(argument, type_of, mro=False, **kwargs):
         if kwargs.pop("check_duck_typing", None):
             return check_duck_typing(argument, type_of)
 
-        if isinstance(type_of, typing_base_class) or (py_version >= 9 and origin is not None):
-            try:
-                return supported_typings[f"checking_typing_{origin_name}"](
-                    argument, get_possible_types(type_of, origin_name), mro, **kwargs
-                )
-            except AttributeError:
-                return isinstance(argument, type_of.__args__)
+        if "any" in origin_name:
+            return validate_object(argument, kwargs.get("validation_with"))
+        if "json" in origin_name or "json" in str(type_of):
+            return checking_typing_json(argument, type_of, mro)
+
+        if origin is typing.Union:
+            return checking_typing_union(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
+        elif origin in (list, typing.MutableSequence, typing.Deque, deque):
+            if extension_module:
+                return module_checking_typing_list(argument, type_of)
+            return checking_typing_list(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
+        elif origin in (typing.Iterable, Iterable):
+            return checking_typing_iterable(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
+        elif origin is set:
+            if extension_module:
+                return module_checking_typing_set(argument, type_of)
+            return checking_typing_set(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
+        elif origin is tuple:
+            if extension_module:
+                return module_checking_typing_tuple(argument, type_of)
+            return checking_typing_tuple(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
+        elif origin in (
+            dict,
+            typing.MutableMapping,
+            typing.Dict,
+            typing.DefaultDict,
+            typing.OrderedDict,
+            typing.Counter,
+            typing.ChainMap,
+        ):
+            if extension_module:
+                return module_checking_typing_dict(argument, type_of)
+            return checking_typing_dict(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
+        elif origin in (typing.Callable, Callable):
+            return checking_typing_callable(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
+        elif origin is type:
+            return checking_typing_type(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
+        elif origin is Validator:
+            if extension_module:
+                return module_checking_typing_validator(argument, type_of)
+            return checking_typing_validator(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
+        elif origin is IterValidator:
+            return checking_typing_itervalidator(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
+        elif origin is typing.Literal:
+            return checking_typing_literal(
+                argument, get_possible_types(type_of, origin_name), mro, **kwargs
+            )
         elif isinstance(type_of, str):
             return argument.__class__.__name__ == type_of
-        elif origin_name in ("_typeddictmeta", "matchtypeddict"):
+        elif origin_name in ("_typeddictmeta", "matchtypeddict", "typeddict"):
             return checking_typing_typeddict(argument, get_possible_types(type_of, "typeddict"))
+        elif origin_name == "required":
+            return checking_typing_typeddict_required(
+                argument, get_possible_types(type_of, origin_name)
+            )
+        elif origin_name == "notrequired":
+            return checking_typing_typeddict_notrequired(
+                argument, get_possible_types(type_of, origin_name)
+            )
         elif mro:
             if origin_name == "union":
                 possible_types = get_possible_types(type_of)
@@ -457,7 +496,7 @@ def check_type(argument, type_of, mro=False, **kwargs):
         else:
             try:
                 is_instance = isinstance(argument, type_of) or argument == type_of
-            except TypeError:
+            except (TypeError, AttributeError):
                 return isinstance(argument, type_of._subs_tree()[1:])
             else:
                 if not is_instance:
