@@ -10,8 +10,6 @@ import weakref
 from functools import partial
 from typing import Any, _GenericAlias, _SpecialForm, _type_repr
 
-from strongtyping.strong_typing_utils import py_version
-
 
 class _Validator(_GenericAlias, _root=True):  # type: ignore
     _name = "Validator"
@@ -63,37 +61,30 @@ class _IterValidator(_GenericAlias, _root=True):  # type: ignore
         return f"strong_typing_utils.IterValidator[{_type_repr(args[0])}, {func_info}]"
 
 
-if py_version >= 9:
+@_SpecialForm  # type: ignore
+def Validator(self, parameters, *args, **kwargs):
+    if isinstance(parameters, _GenericAlias):
+        raise TypeError("Validator needs min 2 values. Validator[type, function]")
+    if not parameters:
+        raise TypeError("Cannot take a Validator of no type/function.")
+    if len(parameters) > 3:
+        raise TypeError("Validator takes max 3 values.")
+    if not inspect.isfunction(parameters[1]) and not isinstance(parameters[1], partial):
+        raise TypeError("Validator[..., arg]: arg should be a function.")
+    return _Validator(self, parameters)
 
-    @_SpecialForm  # type: ignore
-    def Validator(self, parameters, *args, **kwargs):
-        if isinstance(parameters, _GenericAlias):
-            raise TypeError("Validator needs min 2 values. Validator[type, function]")
-        if not parameters:
-            raise TypeError("Cannot take a Validator of no type/function.")
-        if len(parameters) > 3:
-            raise TypeError("Validator takes max 3 values.")
-        if not inspect.isfunction(parameters[1]) and not isinstance(parameters[1], partial):
-            raise TypeError("Validator[..., arg]: arg should be a function.")
-        return _Validator(self, parameters)
 
-    @_SpecialForm  # type: ignore
-    def IterValidator(self, parameters, *args, **kwargs):
-        if isinstance(parameters, _GenericAlias):
-            raise TypeError("Validator needs min 2 values. Validator[type, function]")
-        if not parameters:
-            raise TypeError("Cannot take a Validator of no type/function.")
-        if len(parameters) > 3:
-            raise TypeError("Validator takes max 3 values.")
-        if not inspect.isfunction(parameters[1]) and not isinstance(parameters[1], partial):
-            raise TypeError("Validator[..., arg]: arg should be a function.")
-        return _IterValidator(self, parameters)
-
-else:
-    from typing import KT, VT, _alias, _GenericAlias, _type_repr  # type: ignore
-
-    Validator = _alias(_Validator, (KT, VT), inst=False)
-    IterValidator = _alias(_IterValidator, (KT, VT), inst=False)
+@_SpecialForm  # type: ignore
+def IterValidator(self, parameters, *args, **kwargs):
+    if isinstance(parameters, _GenericAlias):
+        raise TypeError("Validator needs min 2 values. Validator[type, function]")
+    if not parameters:
+        raise TypeError("Cannot take a Validator of no type/function.")
+    if len(parameters) > 3:
+        raise TypeError("Validator takes max 3 values.")
+    if not inspect.isfunction(parameters[1]) and not isinstance(parameters[1], partial):
+        raise TypeError("Validator[..., arg]: arg should be a function.")
+    return _IterValidator(self, parameters)
 
 
 class FrozenType:
